/*
 * files2pdb.java
 *      Convert a pilot pdb database to files.
 *	Usage: java files2pdb foo.pdb bar
 *
 * Ken Shirriff         ken.shirriff@eng.sun.com
 *
 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 *
 * Please refer to the file "license.txt"
 * for further important copyright and licensing information.
 */

import java.io.*;
import java.util.*;

public class files2pdb {
  public static void main(String args[]) {
    try {
      String infofile = null;
      byte name[] = new byte[32];
      short attributes=(short)0x8000;
      short version=1;
      int creationDate=0xb08823ad;
      int modificationDate=0xb08823ad;
      int lastBackupDate=0xb08823ad;
      int modificationNumber=1;
      int appInfoID=0;
      int sortInfoID=0;
      byte type[] = new byte[4];
      byte creator[] = new byte[4];
      int uniqueIDSeed=0;
      int argsidx;
      for (argsidx=0; argsidx<args.length-2; argsidx+=2) {
	if (args[argsidx].equals("-argsidx")) {
	  infofile = args[argsidx+1];
	  appInfoID = (int)(new File(infofile)).length();
	} else if (args[argsidx].equals("-n")) {
	  int l = args[argsidx+1].length();
	  args[argsidx+1].getBytes(0, l, name, 0);
	  name[l] = '\0';
	} else if (args[argsidx].equals("-a")) {
	  attributes = (short)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].equals("-v")) {
	  version = (short)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].equals("-cd")) {
	  creationDate = (int)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].equals("-md")) {
	  modificationDate = (int)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].equals("-bd")) {
	  lastBackupDate = (int)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].equals("-mn")) {
	  modificationNumber = (int)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].equals("-t")) {
	  args[argsidx+1].getBytes(0, 4, type, 0);
	} else if (args[argsidx].equals("-c")) {
	  args[argsidx+1].getBytes(0, 4, creator, 0);
	} else if (args[argsidx].equals("-s")) {
	  uniqueIDSeed = (int)Integer.parseInt(args[argsidx+1]);
	} else if (args[argsidx].charAt(0) == '-') {
	  usage();
	} else {
	  break;
	}
      }

      if (args.length-argsidx<2) {
	usage();
      }

      String outfile = args[argsidx];
      short numRecords;
      argsidx++;
      DataOutputStream os = new DataOutputStream(new FileOutputStream(outfile));
      numRecords = (short)(args.length-argsidx);

      int offset = 80+numRecords*8;

      if (infofile != null) {
	appInfoID += offset;
	offset = appInfoID;
      }

      // DatabaseHdrType
      os.write(name);
      os.writeShort(attributes);
      os.writeShort(version);
      os.writeInt(creationDate);
      os.writeInt(modificationDate);
      os.writeInt(lastBackupDate);
      os.writeInt(modificationNumber);
      os.writeInt(appInfoID);
      os.writeInt(sortInfoID);
      os.write(type);
      os.write(creator);
      os.writeInt(uniqueIDSeed);

      // RecordListType
      int nextRecordListID = 0;
      os.writeInt(nextRecordListID);
      os.writeShort(numRecords);
      for (int i=0; i<numRecords; i++) {
	os.writeInt(offset); // LocalChunkID
	os.writeByte(0); // attributes
	os.writeByte((byte)(i>>16)); // uniqueID
	os.writeByte((byte)(i>>8)); // uniqueID
	os.writeByte((byte)(i>>0)); // uniqueID
	offset += (new File(args[argsidx+i])).length();
      }
      os.writeShort(0); // pad
      if (infofile != null) {
        writeFile(os, infofile);
      }
      for (int i=0; i<numRecords; i++) {
	writeFile(os, args[argsidx+i]);
      }
      os.close();

    } catch (IOException e) {
      System.out.println("IOException "+e);
    }
  }

  static void writeFile(DataOutputStream os, String name) throws
      IOException {
    byte buf[] = new byte[1024];
    DataInputStream is = new DataInputStream(new FileInputStream(name));
    try {
      while (true) {
	int read = is.read(buf);
	if (read < 0) {
	  break;
	}
	os.write(buf, 0, read);
      }
    } catch (EOFException e) {}
    is.close();
  }

  static void usage() {
    System.out.println("Usage: java files2pdb [-i appInfoFile] [-n name] [-a attributes] [-v version] [-cd creationdate] [-md modificationdate] [-bd backupDate] [-mn modificationNumber] [-t type ] [-c creator] [-s seed] outfile recordFiles(s)");
    System.exit(-1);
  }
}
